@@ -2,6 +2,7 @@ package Facebook::InstantArticle;
 use Moose;
 use namespace::autoclean;
 
+use Mojo::Log;
 use XML::Generator;
 
 use Facebook::InstantArticle::Analytics;
@@ -16,7 +17,7 @@ use Facebook::InstantArticle::List;
 use Facebook::InstantArticle::Map;
 use Facebook::InstantArticle::Paragraph;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =encoding utf-8
 
@@ -27,8 +28,8 @@ markup.
 
 =head1 DESCRIPTION
 
-Facebook::InstantArticle is a simple helper class for generating Facebook
-Instant Articles markup.
+Facebook::InstantArticle is a simple helper class for generating L<Facebook
+Instant Articles markup|https://developers.facebook.com/docs/instant-articles/reference>.
 
 At the moment it doesn't support all of the features, and both the internal and
 external API are subject to change in upcoming releases, so use with care.
@@ -111,6 +112,8 @@ has '_body_elements'   => ( isa => 'ArrayRef[Object]', is => 'ro', default => su
 has '_footer_elements' => ( isa => 'ArrayRef[Object]', is => 'ro', default => sub { [] } );
 has '_credit_elements' => ( isa => 'ArrayRef[Object]', is => 'ro', default => sub { [] } );
 
+has '_log' => ( isa => 'Mojo::Log', is => 'ro', default => sub { Mojo::Log->new } );
+
 =head1 METHODS
 
 =head2 add_lead_asset_image
@@ -127,7 +130,13 @@ Adds a lead asset image to the article.
 sub add_lead_asset_image {
     my $self = shift;
 
-    push( @{$self->_header_elements}, Facebook::InstantArticle::Figure::Image->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Figure::Image->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_header_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add lead asset image to article header!' );
 }
 
 =head2 add_lead_asset_video
@@ -144,7 +153,13 @@ Adds a lead asset video to the article.
 sub add_lead_asset_video {
     my $self = shift;
 
-    push( @{$self->_header_elements}, Facebook::InstantArticle::Figure::Video->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Figure::Video->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_header_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add lead asset video to article header!' );
 }
 
 =head2 add_author
@@ -160,7 +175,13 @@ Adds an author to the article.
 sub add_author {
     my $self = shift;
 
-    push( @{$self->_header_elements}, Facebook::InstantArticle::Author->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Author->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_header_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add author to article header!' );
 }
 
 =head2 add_paragraph
@@ -174,11 +195,13 @@ Adds a paragraph to the article.
 sub add_paragraph {
     my $self = shift;
 
-    my $p = Facebook::InstantArticle::Paragraph->new( @_ );
-
-    if ( $p->is_valid ) {
-        push( @{$self->_body_elements}, $p );
+    if ( my $e = Facebook::InstantArticle::Paragraph->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
     }
+
+    $self->_log->warn( 'Failed to add paragraph to article body!' );
 }
 
 =head2 add_image
@@ -195,7 +218,13 @@ Adds an image to the article.
 sub add_image {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Figure::Image->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Figure::Image->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add image to article body!' );
 }
 
 =head2 add_video
@@ -212,7 +241,13 @@ Adds a video to the article.
 sub add_video {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Figure::Video->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Figure::Video->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add video to article body!' );
 }
 
 =head2 add_slideshow
@@ -239,7 +274,11 @@ sub add_slideshow {
     my $self      = shift;
     my $slideshow = shift;
 
-    push( @{$self->_body_elements}, $slideshow );
+    if ( defined $slideshow && $slideshow->is_valid ) {
+        return push( @{$self->_body_elements}, $slideshow );
+    }
+
+    $self->_log->warn( 'Failed to add slideshow to article body!' );
 }
 
 =head2 add_credit
@@ -253,7 +292,13 @@ Adds a credit to the article.
 sub add_credit {
     my $self = shift;
 
-    push( @{$self->_credit_elements}, Facebook::InstantArticle::Paragraph->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Paragraph->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_credit_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add credit to article credits!' );
 }
 
 =head2 add_copyright
@@ -267,7 +312,13 @@ Adds a copyright to the article.
 sub add_copyright {
     my $self = shift;
 
-    push( @{$self->_footer_elements}, Facebook::InstantArticle::Copyright->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Copyright->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_footer_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add copyright to article footer!' );
 }
 
 =head2 add_list
@@ -285,7 +336,13 @@ Adds a Facebook::InstantArticle::List object to the article.
 sub add_list {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::List->new(@_) );
+    if ( my $e = Facebook::InstantArticle::List->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add list to article body!' );
 }
 
 =head2 add_blockquote
@@ -299,7 +356,13 @@ Adds a blockquote to the article.
 sub add_blockquote {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Blockquote->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Blockquote->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add blockquote to article body!' );
 }
 
 =head2 add_embed
@@ -313,7 +376,13 @@ Adds an embed to the article.
 sub add_embed {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Embed->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Embed->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add embed to article body!' );
 }
 
 =head2 add_heading
@@ -330,14 +399,20 @@ Adds a heading to the article BODY.
 sub add_heading {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Heading->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Heading->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add heading to article body!' );
 }
 
 =head2 add_map
 
 Adds a map to the article BODY.
 
-    $ia->add_heading(
+    $ia->add_map(
         latitude  => 56.1341342,
         longitude => 23.253474,
     );
@@ -347,7 +422,13 @@ Adds a map to the article BODY.
 sub add_map {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Map->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Map->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add map to article body!' );
 }
 
 =head2 add_analytics
@@ -359,7 +440,13 @@ Adds an analytics iframe to the article body.
 sub add_analytics {
     my $self = shift;
 
-    push( @{$self->_body_elements}, Facebook::InstantArticle::Analytics->new(@_) );
+    if ( my $e = Facebook::InstantArticle::Analytics->new(@_) ) {
+        if ( $e->is_valid ) {
+            return push( @{$self->_body_elements}, $e );
+        }
+    }
+
+    $self->_log->warn( 'Failed to add analytics/tracking to article body!' );
 }
 
 =head2 to_string
@@ -424,7 +511,7 @@ Copyright 2016- Tore Aursand
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
 
 =cut
